@@ -6,7 +6,7 @@ use FCGI::Engine::Manager::Server;
 
 use Config::Any;
 
-our $VERSION   = '0.03';
+our $VERSION   = '0.06';
 our $AUTHORITY = 'cpan:STEVAN';
 
 with 'MooseX::Getopt';
@@ -64,13 +64,20 @@ sub start {
 
     foreach my $server ( @servers ) {
     
+        if (-e $server->pidfile) {
+            my $pid = $server->pid_obj;
+            if ($pid->is_running) {
+                $self->log("Pid " . $pid->pid . " is already running");
+                return;
+            }
+            $server->remove_pid_object;
+        }
+    
         my @cli = $server->construct_command_line();
         $self->log("Running @cli");
     
         unless (system(@cli) == 0) {
             $self->log("Could not execute command (@cli) exited with status $?");
-            $self->log("... stoping FCGI servers");
-            $self->stop;
             return;
         }
     
@@ -135,12 +142,13 @@ sub stop {
             
             $self->log("Killing PID " . $pid->pid . " from $$ ");
             kill TERM => $pid->pid;
-            
+
             while ($pid->is_running) {
                 $self->log("pid (" . $server->pidfile . ") is still running, sleeping ...");
                 sleep 1;
             } 
             
+            $server->pid_obj->remove();
             $server->remove_pid_obj;
         }
     
@@ -160,6 +168,30 @@ sub restart {
     $self->start( @_ );
 }
 
+
+sub graceful {
+    my $self = shift;
+    my @servers = (@_ && defined $_[0]) ? $self->_find_server_by_name( @_ ) : @{ $self->servers };
+    my @pids;
+    foreach my $server ( @servers ) {
+        push @pids, $server->pid_obj->pid;
+        unlink($server->pidfile);
+        $server->pid_obj->remove_pid_obj();
+    }
+    $self->start( @_ );
+    foreach my $pid ( @pids ) {
+        $self->log("... Killing old fcgi process $pid");
+        kill TERM => $pid;
+    }
+    foreach my $server ( @servers ) {
+        while (-f $server->pidfile) {
+            $self->log("pid (" . $server->pidfile . ") has not been removed, sleeping ...");
+            sleep 1;
+        }
+        $server->pid_obj->write();
+    }
+}
+
 sub _find_server_by_name {
     my( $self, @names ) = @_;
 
@@ -193,6 +225,7 @@ FCGI::Engine::Manager - Manage multiple FCGI::Engine instances
   $m->start($server_name)        if $command eq 'start';
   $m->stop($server_name)         if $command eq 'stop';  
   $m->restart($server_name)      if $command eq 'restart';
+  $m->graceful($server_name)      if $command eq 'graceful';
   print $m->status($server_name) if $command eq 'status';     
 
   # on the command line
@@ -252,7 +285,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2007-2008 by Infinity Interactive, Inc.
+Copyright 2007-2009 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
